@@ -12,7 +12,7 @@ pub use vertex::{Point, Vertex2D};
 
 use super::{
     Color, Command, Draw, DrawList, DrawState, Geometry, GeometryVariants, LineState, LineVertex,
-    Projection, TextureCache, Transform3D,
+    Projection,
 };
 use solstice::texture::Texture;
 
@@ -21,39 +21,55 @@ where
     G: crate::GeometryKind<'a, crate::Vertex2D> + 'a,
 {
     fn draw(&mut self, geometry: G) {
-        self.draw_with_color_and_transform(geometry, self.color, self.transform)
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            self.color,
+            self.transform,
+            None,
+        );
     }
 
-    fn draw_with_transform<TX: Into<Transform3D>>(&mut self, geometry: G, transform: TX) {
-        self.draw_with_color_and_transform(geometry, self.color, transform)
+    fn draw_with_transform<TX>(&mut self, geometry: G, transform: TX)
+    where
+        TX: Into<mint::ColumnMatrix4<f32>>,
+    {
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            self.color,
+            transform.into(),
+            None,
+        );
     }
 
     fn draw_with_color<C: Into<Color>>(&mut self, geometry: G, color: C) {
-        self.draw_with_color_and_transform(geometry, color, self.transform)
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            color.into(),
+            self.transform,
+            None,
+        );
     }
 
-    fn draw_with_color_and_transform<C: Into<Color>, TX: Into<Transform3D>>(
-        &mut self,
-        geometry: G,
-        color: C,
-        transform: TX,
-    ) {
-        self.commands.push(Command::Draw(DrawState {
-            data: GeometryVariants::D2(geometry.into()),
-            transform: transform.into(),
-            camera: self.camera,
-            projection_mode: self
-                .projection_mode
-                .unwrap_or(Projection::Orthographic(None)),
-            color: color.into(),
-            texture: None,
-            target: self.target.clone(),
-            shader: self.shader.clone(),
-        }))
+    fn draw_with_color_and_transform<C, TX>(&mut self, geometry: G, color: C, transform: TX)
+    where
+        C: Into<Color>,
+        TX: Into<mint::ColumnMatrix4<f32>>,
+    {
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            color.into(),
+            transform.into(),
+            None,
+        );
     }
 
     fn image<T: Texture>(&mut self, geometry: G, texture: T) {
-        self.image_with_color_and_transform(geometry, texture, self.color, self.transform)
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            self.color,
+            self.transform,
+            Some(texture.into()),
+        );
     }
 
     fn image_with_color<T, C>(&mut self, geometry: G, texture: T, color: C)
@@ -61,15 +77,25 @@ where
         T: Texture,
         C: Into<Color>,
     {
-        self.image_with_color_and_transform(geometry, texture, color, self.transform)
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            color.into(),
+            self.transform,
+            Some(texture.into()),
+        );
     }
 
     fn image_with_transform<T, TX>(&mut self, geometry: G, texture: T, transform: TX)
     where
         T: Texture,
-        TX: Into<Transform3D>,
+        TX: Into<mint::ColumnMatrix4<f32>>,
     {
-        self.image_with_color_and_transform(geometry, texture, self.color, transform)
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            self.color,
+            transform.into(),
+            Some(texture.into()),
+        );
     }
 
     fn image_with_color_and_transform<T, C, TX>(
@@ -81,24 +107,14 @@ where
     ) where
         T: Texture,
         C: Into<Color>,
-        TX: Into<Transform3D>,
+        TX: Into<mint::ColumnMatrix4<f32>>,
     {
-        self.commands.push(Command::Draw(DrawState {
-            data: GeometryVariants::D2(geometry.into()),
-            transform: transform.into(),
-            camera: self.camera,
-            projection_mode: self
-                .projection_mode
-                .unwrap_or(Projection::Orthographic(None)),
-            color: color.into(),
-            texture: Some(TextureCache {
-                ty: texture.get_texture_type(),
-                key: texture.get_texture_key(),
-                info: texture.get_texture_info(),
-            }),
-            target: self.target.clone(),
-            shader: self.shader.clone(),
-        }))
+        self.push_draw(
+            GeometryVariants::D2(geometry.into()),
+            color.into(),
+            transform.into(),
+            Some(texture.into()),
+        );
     }
 }
 impl<'a, G> crate::Stroke<crate::Vertex2D, G> for DrawList<'a>
@@ -109,7 +125,10 @@ where
         self.stroke_with_color_and_transform(geometry, self.color, self.transform)
     }
 
-    fn stroke_with_transform<TX: Into<Transform3D>>(&mut self, geometry: G, transform: TX) {
+    fn stroke_with_transform<TX>(&mut self, geometry: G, transform: TX)
+    where
+        TX: Into<mint::ColumnMatrix4<f32>>,
+    {
         self.stroke_with_color_and_transform(geometry, self.color, transform)
     }
 
@@ -117,12 +136,11 @@ where
         self.stroke_with_color_and_transform(geometry, color, self.transform)
     }
 
-    fn stroke_with_color_and_transform<C: Into<Color>, TX: Into<Transform3D>>(
-        &mut self,
-        geometry: G,
-        color: C,
-        transform: TX,
-    ) {
+    fn stroke_with_color_and_transform<C, TX>(&mut self, geometry: G, color: C, transform: TX)
+    where
+        C: Into<Color>,
+        TX: Into<mint::ColumnMatrix4<f32>>,
+    {
         let crate::Geometry { vertices, .. } = geometry.into();
         self.commands.push(Command::Line(DrawState {
             data: LineState {
@@ -130,7 +148,7 @@ where
                     .iter()
                     .map(|v: &Vertex2D| LineVertex {
                         position: [v.position[0], v.position[1], 0.],
-                        width: 5.0,
+                        width: self.line_width,
                         color: [1., 1., 1., 1.],
                     })
                     .collect::<Vec<_>>()
@@ -160,10 +178,10 @@ pub trait SimpleConvexGeometry: std::fmt::Debug {
 macro_rules! impl_array_simple_convex_geom {
     ($ty:ty, $count:expr) => {
         impl SimpleConvexGeometry for [$ty; $count] {
-            type Vertices = std::iter::Map<std::vec::IntoIter<$ty>, fn($ty) -> Vertex2D>;
+            type Vertices = std::iter::Map<std::array::IntoIter<$ty, $count>, fn($ty) -> Vertex2D>;
 
             fn vertices(&self) -> Self::Vertices {
-                self.to_vec().into_iter().map(Into::into)
+                std::array::IntoIter::new(*self).map(Into::into)
             }
 
             fn vertex_count(&self) -> usize {
@@ -172,10 +190,10 @@ macro_rules! impl_array_simple_convex_geom {
         }
 
         impl SimpleConvexGeometry for &[$ty; $count] {
-            type Vertices = std::iter::Map<std::vec::IntoIter<$ty>, fn($ty) -> Vertex2D>;
+            type Vertices = std::iter::Map<std::array::IntoIter<$ty, $count>, fn($ty) -> Vertex2D>;
 
             fn vertices(&self) -> Self::Vertices {
-                self.to_vec().into_iter().map(Into::into)
+                std::array::IntoIter::new(**self).map(Into::into)
             }
 
             fn vertex_count(&self) -> usize {
