@@ -174,14 +174,8 @@ impl Graphics {
                                 viewport,
                                 target.is_some(),
                             );
-                            shader.send_uniform(
-                                "uView",
-                                solstice::shader::RawUniformValue::Mat4(camera.into()),
-                            );
-                            shader.send_uniform(
-                                "uModel",
-                                solstice::shader::RawUniformValue::Mat4(transform.into()),
-                            );
+                            shader.set_view(camera);
+                            shader.set_model(*transform);
                             shader.set_color(*color);
                             match texture.as_ref() {
                                 None => shader.bind_texture(&self.default_texture),
@@ -213,14 +207,8 @@ impl Graphics {
                                 viewport,
                                 target.is_some(),
                             );
-                            shader.send_uniform(
-                                "uView",
-                                solstice::shader::RawUniformValue::Mat4(camera.into()),
-                            );
-                            shader.send_uniform(
-                                "uModel",
-                                solstice::shader::RawUniformValue::Mat4(transform.into()),
-                            );
+                            shader.set_view(camera);
+                            shader.set_model(*transform);
                             shader.set_color(draw_state.color);
                             match texture.as_ref() {
                                 None => shader.bind_texture(&self.default_texture),
@@ -282,14 +270,8 @@ impl Graphics {
                         viewport,
                         target.is_some(),
                     );
-                    shader.send_uniform(
-                        "uView",
-                        solstice::shader::RawUniformValue::Mat4(camera.into()),
-                    );
-                    shader.send_uniform(
-                        "uModel",
-                        solstice::shader::RawUniformValue::Mat4(transform.into()),
-                    );
+                    shader.set_view(camera);
+                    shader.set_model(*transform);
                     match texture.as_ref() {
                         None => shader.bind_texture(&self.default_texture),
                         Some(texture) => shader.bind_texture(texture),
@@ -331,6 +313,7 @@ impl Graphics {
                                 font_id,
                                 scale,
                                 bounds,
+                                layout,
                             },
                         transform,
                         camera,
@@ -351,6 +334,7 @@ impl Graphics {
                             },
                         },
                         *bounds,
+                        layout.into(),
                         ctx,
                     );
 
@@ -370,14 +354,8 @@ impl Graphics {
                         viewport,
                         target.is_some(),
                     );
-                    shader.send_uniform(
-                        "uView",
-                        solstice::shader::RawUniformValue::Mat4(camera.into()),
-                    );
-                    shader.send_uniform(
-                        "uModel",
-                        solstice::shader::RawUniformValue::Mat4(transform.into()),
-                    );
+                    shader.set_view(camera);
+                    shader.set_model(*transform);
                     shader.set_color(Color::new(1., 1., 1., 1.));
                     shader.activate(ctx);
 
@@ -469,14 +447,14 @@ impl<'a, V, I> Geometry<'a, V, I> {
 
 pub trait Draw<V: solstice::vertex::Vertex, G> {
     fn draw(&mut self, geometry: G);
-    fn draw_with_transform<TX: Into<d3::Transform3D>>(&mut self, geometry: G, transform: TX);
+    fn draw_with_transform<TX>(&mut self, geometry: G, transform: TX)
+    where
+        TX: Into<mint::ColumnMatrix4<f32>>;
     fn draw_with_color<C: Into<Color>>(&mut self, geometry: G, color: C);
-    fn draw_with_color_and_transform<C: Into<Color>, TX: Into<d3::Transform3D>>(
-        &mut self,
-        geometry: G,
-        color: C,
-        transform: TX,
-    );
+    fn draw_with_color_and_transform<C, TX>(&mut self, geometry: G, color: C, transform: TX)
+    where
+        C: Into<Color>,
+        TX: Into<mint::ColumnMatrix4<f32>>;
     fn image<T: Texture>(&mut self, geometry: G, texture: T);
     fn image_with_color<T, C>(&mut self, geometry: G, texture: T, color: C)
     where
@@ -485,7 +463,7 @@ pub trait Draw<V: solstice::vertex::Vertex, G> {
     fn image_with_transform<T, TX>(&mut self, geometry: G, texture: T, transform: TX)
     where
         T: Texture,
-        TX: Into<d3::Transform3D>;
+        TX: Into<mint::ColumnMatrix4<f32>>;
     fn image_with_color_and_transform<T, C, TX>(
         &mut self,
         geometry: G,
@@ -495,18 +473,18 @@ pub trait Draw<V: solstice::vertex::Vertex, G> {
     ) where
         T: Texture,
         C: Into<Color>,
-        TX: Into<d3::Transform3D>;
+        TX: Into<mint::ColumnMatrix4<f32>>;
 }
 pub trait Stroke<V: solstice::vertex::Vertex, G> {
     fn stroke(&mut self, geometry: G);
-    fn stroke_with_transform<TX: Into<d3::Transform3D>>(&mut self, geometry: G, transform: TX);
+    fn stroke_with_transform<TX>(&mut self, geometry: G, transform: TX)
+    where
+        TX: Into<mint::ColumnMatrix4<f32>>;
     fn stroke_with_color<C: Into<Color>>(&mut self, geometry: G, color: C);
-    fn stroke_with_color_and_transform<C: Into<Color>, TX: Into<d3::Transform3D>>(
-        &mut self,
-        geometry: G,
-        color: C,
-        transform: TX,
-    );
+    fn stroke_with_color_and_transform<C, TX>(&mut self, geometry: G, color: C, transform: TX)
+    where
+        C: Into<Color>,
+        TX: Into<mint::ColumnMatrix4<f32>>;
 }
 
 #[derive(PartialEq, Clone, Debug)]
@@ -516,6 +494,19 @@ struct TextureCache {
     info: solstice::texture::TextureInfo,
 }
 
+impl<T> From<T> for TextureCache
+where
+    T: solstice::texture::Texture,
+{
+    fn from(texture: T) -> Self {
+        TextureCache {
+            ty: texture.get_texture_type(),
+            key: texture.get_texture_key(),
+            info: texture.get_texture_info(),
+        }
+    }
+}
+
 impl solstice::texture::Texture for &TextureCache {
     fn get_texture_key(&self) -> solstice::TextureKey {
         self.key
@@ -729,7 +720,7 @@ where
 #[derive(Clone, Debug)]
 pub struct DrawState<T> {
     data: T,
-    transform: Transform3D,
+    transform: mint::ColumnMatrix4<f32>,
     camera: Transform3D,
     projection_mode: Projection,
     color: Color,
@@ -745,12 +736,85 @@ pub struct LineState<'a> {
     depth_buffer: bool,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum HorizontalAlign {
+    Left,
+    Center,
+    Right,
+}
+
+impl From<&HorizontalAlign> for glyph_brush::HorizontalAlign {
+    fn from(align: &HorizontalAlign) -> Self {
+        match align {
+            HorizontalAlign::Left => glyph_brush::HorizontalAlign::Left,
+            HorizontalAlign::Center => glyph_brush::HorizontalAlign::Center,
+            HorizontalAlign::Right => glyph_brush::HorizontalAlign::Right,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum VerticalAlign {
+    Top,
+    Center,
+    Bottom,
+}
+
+impl From<&VerticalAlign> for glyph_brush::VerticalAlign {
+    fn from(align: &VerticalAlign) -> Self {
+        match align {
+            VerticalAlign::Top => glyph_brush::VerticalAlign::Top,
+            VerticalAlign::Center => glyph_brush::VerticalAlign::Center,
+            VerticalAlign::Bottom => glyph_brush::VerticalAlign::Bottom,
+        }
+    }
+}
+
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
+pub enum PrintLayout {
+    SingleLine {
+        h_align: HorizontalAlign,
+        v_align: VerticalAlign,
+    },
+    Wrap {
+        h_align: HorizontalAlign,
+        v_align: VerticalAlign,
+    },
+}
+
+impl std::default::Default for PrintLayout {
+    fn default() -> Self {
+        PrintLayout::Wrap {
+            h_align: HorizontalAlign::Left,
+            v_align: VerticalAlign::Top,
+        }
+    }
+}
+
+impl From<&PrintLayout> for glyph_brush::Layout<glyph_brush::BuiltInLineBreaker> {
+    fn from(layout: &PrintLayout) -> Self {
+        match layout {
+            PrintLayout::SingleLine { h_align, v_align } => glyph_brush::Layout::SingleLine {
+                line_breaker: glyph_brush::BuiltInLineBreaker::default(),
+                h_align: h_align.into(),
+                v_align: v_align.into(),
+            },
+            PrintLayout::Wrap { h_align, v_align } => glyph_brush::Layout::Wrap {
+                line_breaker: glyph_brush::BuiltInLineBreaker::default(),
+                h_align: h_align.into(),
+                v_align: v_align.into(),
+            },
+        }
+    }
+}
+
 #[derive(Clone, Debug)]
 pub struct PrintState<'a> {
     text: std::borrow::Cow<'a, str>,
     font_id: glyph_brush::FontId,
     scale: f32,
     bounds: d2::Rectangle,
+    layout: PrintLayout,
 }
 
 #[derive(Clone, Debug)]
@@ -761,18 +825,55 @@ pub enum Command<'a> {
     Clear(Color, Option<Canvas>),
 }
 
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct DrawList<'a> {
     commands: Vec<Command<'a>>,
     color: Color,
-    transform: Transform3D,
+    transform: mint::ColumnMatrix4<f32>,
+    line_width: f32,
     camera: Transform3D,
     projection_mode: Option<Projection>,
     target: Option<Canvas>,
     shader: Option<Shader>,
 }
 
+impl Default for DrawList<'_> {
+    fn default() -> Self {
+        Self {
+            commands: vec![],
+            color: Default::default(),
+            transform: Transform3D::default().into(),
+            line_width: 1.0,
+            camera: Default::default(),
+            projection_mode: None,
+            target: None,
+            shader: None,
+        }
+    }
+}
+
 impl<'a> DrawList<'a> {
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    pub fn new_from_state<'b>(other: &DrawList) -> DrawList<'b> {
+        DrawList {
+            commands: vec![],
+            color: other.color,
+            transform: other.transform,
+            line_width: 1.0,
+            camera: other.camera,
+            projection_mode: other.projection_mode,
+            target: other.target.clone(),
+            shader: other.shader.clone(),
+        }
+    }
+
+    pub fn append(&mut self, other: &mut Self) {
+        self.commands.append(&mut other.commands);
+    }
+
     pub fn clear<C: Into<Color>>(&mut self, color: C) {
         let command = Command::Clear(color.into(), self.target.clone());
         self.commands.push(command)
@@ -788,8 +889,40 @@ impl<'a> DrawList<'a> {
                 font_id,
                 scale,
                 bounds,
+                layout: Default::default(),
             },
-            transform: self.transform,
+            transform: self.transform.into(),
+            camera: self.camera,
+            projection_mode: self
+                .projection_mode
+                .unwrap_or(Projection::Orthographic(None)),
+            color: self.color,
+            texture: None,
+            target: self.target.clone(),
+            shader: self.shader.clone(),
+        });
+        self.commands.push(command);
+    }
+
+    pub fn print_with_layout<T>(
+        &mut self,
+        text: T,
+        font_id: glyph_brush::FontId,
+        scale: f32,
+        bounds: Rectangle,
+        layout: PrintLayout,
+    ) where
+        T: Into<std::borrow::Cow<'a, str>>,
+    {
+        let command = Command::Print(DrawState {
+            data: PrintState {
+                text: text.into(),
+                font_id,
+                scale,
+                bounds,
+                layout,
+            },
+            transform: self.transform.into(),
             camera: self.camera,
             projection_mode: self
                 .projection_mode
@@ -812,7 +945,7 @@ impl<'a> DrawList<'a> {
                 is_loop: false,
                 depth_buffer: false,
             },
-            transform: self.transform,
+            transform: self.transform.into(),
             camera: self.camera,
             projection_mode: self
                 .projection_mode
@@ -835,7 +968,7 @@ impl<'a> DrawList<'a> {
                 is_loop: false,
                 depth_buffer: true,
             },
-            transform: self.transform,
+            transform: self.transform.into(),
             camera: self.camera,
             projection_mode: self
                 .projection_mode
@@ -852,12 +985,12 @@ impl<'a> DrawList<'a> {
         self.color = color.into();
     }
 
-    pub fn set_transform<T: Into<Transform3D>>(&mut self, transform: T) {
+    pub fn set_transform<T: Into<mint::ColumnMatrix4<f32>>>(&mut self, transform: T) {
         self.transform = transform.into();
     }
 
-    pub fn get_transform(&self) -> Transform3D {
-        self.transform
+    pub fn set_line_width(&mut self, line_width: f32) {
+        self.line_width = line_width;
     }
 
     pub fn set_camera<T: Into<Transform3D>>(&mut self, camera: T) {
@@ -877,6 +1010,31 @@ impl<'a> DrawList<'a> {
     }
 }
 
+impl<'a> DrawList<'a> {
+    fn push_draw(
+        &mut self,
+        data: GeometryVariants<'a>,
+        color: Color,
+        transform: mint::ColumnMatrix4<f32>,
+        texture: Option<TextureCache>,
+    ) {
+        let projection_mode = self.projection_mode.unwrap_or_else(|| match &data {
+            GeometryVariants::D2(_) => Projection::Orthographic(None),
+            GeometryVariants::D3(_) => Projection::Perspective(None),
+        });
+        self.commands.push(Command::Draw(DrawState {
+            data,
+            transform,
+            camera: Default::default(),
+            projection_mode,
+            color,
+            texture,
+            target: self.target.clone(),
+            shader: self.shader.clone(),
+        }))
+    }
+}
+
 type ImageResult = Result<solstice::image::Image, solstice::GraphicsError>;
 
 pub fn create_default_texture(gl: &mut solstice::Context) -> ImageResult {
