@@ -78,17 +78,17 @@ struct TextureCache {
 
 const MAX_TEXTURE_UNITS: usize = 8;
 
-#[allow(unused)]
 #[derive(Debug, Clone, PartialEq)]
 pub struct Shader {
     inner: solstice::shader::DynamicShader,
 
-    projection_location: UniformLocation,
+    projection_location: Option<UniformLocation>,
     projection_cache: mint::ColumnMatrix4<f32>,
-    view_location: UniformLocation,
+    view_location: Option<UniformLocation>,
     view_cache: mint::ColumnMatrix4<f32>,
-    model_location: UniformLocation,
+    model_location: Option<UniformLocation>,
     model_cache: mint::ColumnMatrix4<f32>,
+    normal_matrix_location: Option<UniformLocation>,
     color_location: Option<UniformLocation>,
     color_cache: mint::Vector4<f32>,
     resolution_location: Option<UniformLocation>,
@@ -142,6 +142,7 @@ attribute vec2 uv;
 uniform mat4 uProjection;
 uniform mat4 uView;
 uniform mat4 uModel;
+uniform mat4 uNormalMatrix;
 
 {vertex}
 
@@ -183,9 +184,10 @@ impl Shader {
         let shader = DynamicShader::new(ctx, vertex.as_str(), fragment.as_str())
             .map_err(ShaderError::GraphicsError)?;
 
-        let projection_location = get_location(&shader, "uProjection")?;
-        let view_location = get_location(&shader, "uView")?;
-        let model_location = get_location(&shader, "uModel")?;
+        let projection_location = get_location(&shader, "uProjection").ok();
+        let view_location = get_location(&shader, "uView").ok();
+        let model_location = get_location(&shader, "uModel").ok();
+        let normal_matrix_location = get_location(&shader, "uNormalMatrix").ok();
         let color_location = get_location(&shader, "uColor").ok();
         let resolution_location = get_location(&shader, "uResolution").ok();
         let mut textures = (0..MAX_TEXTURE_UNITS).map(|i| {
@@ -218,18 +220,30 @@ impl Shader {
         let projection_cache = identity;
 
         ctx.use_shader(Some(&shader));
-        ctx.set_uniform_by_location(
-            &projection_location,
-            &solstice::shader::RawUniformValue::Mat4(projection_cache),
-        );
-        ctx.set_uniform_by_location(
-            &view_location,
-            &solstice::shader::RawUniformValue::Mat4(identity),
-        );
-        ctx.set_uniform_by_location(
-            &model_location,
-            &solstice::shader::RawUniformValue::Mat4(identity),
-        );
+        if let Some(projection_location) = &projection_location {
+            ctx.set_uniform_by_location(
+                &projection_location,
+                &solstice::shader::RawUniformValue::Mat4(projection_cache),
+            );
+        }
+        if let Some(view_location) = &view_location {
+            ctx.set_uniform_by_location(
+                &view_location,
+                &solstice::shader::RawUniformValue::Mat4(identity),
+            );
+        }
+        if let Some(model_location) = &model_location {
+            ctx.set_uniform_by_location(
+                &model_location,
+                &solstice::shader::RawUniformValue::Mat4(identity),
+            );
+        }
+        if let Some(normal_location) = &normal_matrix_location {
+            ctx.set_uniform_by_location(
+                &normal_location,
+                &solstice::shader::RawUniformValue::Mat4(identity),
+            );
+        }
         if let Some(color_location) = &color_location {
             ctx.set_uniform_by_location(
                 color_location,
@@ -245,6 +259,7 @@ impl Shader {
             view_cache: identity,
             model_location,
             model_cache: identity,
+            normal_matrix_location,
             color_location,
             color_cache: white,
             resolution_location,
@@ -386,6 +401,14 @@ impl Shader {
         }
     }
 
+    pub fn set_view<V: Into<mint::ColumnMatrix4<f32>>>(&mut self, view: V) {
+        self.view_cache = view.into();
+    }
+
+    pub fn set_model<M: Into<mint::ColumnMatrix4<f32>>>(&mut self, model: M) {
+        self.model_cache = model.into();
+    }
+
     pub fn activate(&mut self, ctx: &mut Context) {
         use solstice::shader::RawUniformValue::{Mat4, SignedInt, Vec4};
         ctx.use_shader(Some(&self.inner));
@@ -410,7 +433,23 @@ impl Shader {
                 &solstice::shader::RawUniformValue::Vec4(self.resolution_cache),
             );
         }
-        ctx.set_uniform_by_location(&self.projection_location, &Mat4(self.projection_cache));
+        if let Some(projection_location) = &self.projection_location {
+            ctx.set_uniform_by_location(projection_location, &Mat4(self.projection_cache));
+        }
+        if let Some(view_location) = &self.view_location {
+            ctx.set_uniform_by_location(view_location, &Mat4(self.view_cache));
+        }
+        if let Some(model_location) = &self.model_location {
+            ctx.set_uniform_by_location(model_location, &Mat4(self.model_cache));
+        }
+        if let Some(normal_location) = &self.normal_matrix_location {
+            let v = nalgebra::Matrix4::from(self.view_cache)
+                * nalgebra::Matrix4::from(self.model_cache);
+            if let Some(v) = v.try_inverse() {
+                let v = v.transpose();
+                ctx.set_uniform_by_location(normal_location, &Mat4(v.into()))
+            }
+        }
     }
 }
 
